/* src/track.c - Tracker un user
 * Copyright (C) 2002-2003 Inter System (Cesar@Inter-System.Net)
 *
 * contact: Progs@Inter-System.Net
 *          Cesar@Inter-System.Net
 *          kouak@kouak.org
 * site web: http://coderz.inter-system.net
 *
 * Services pour serveur IRC. Support sur IrcProgs et IrCoderZ
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include <main.h>
#include <time.h>

CVSINFO("$Id: track.c,v 1.5 2003/03/02 14:39:32 romexzf Exp $");

#ifdef HAVE_TRACK
struct track *trackhead = NULL;

struct track *istrack(anUser *user)
{
	struct track *track;
	for(track=trackhead;track && track->tracked != user;track=track->next);
	return track;
}

static void add_track(anUser *nick, aNick *from, long expire)
{
	struct track *new, *current;
	new = (struct track *) calloc(1, sizeof(struct track));
	if(new)
	{
		new->next = NULL;
		new->tracked = nick;
		new->tracker = from;
		new->expire=expire;
		new->from=time(NULL);
		if(!trackhead) trackhead = new;
		else
		{
			current = trackhead;
			while(current->next) current = current->next;
			current->next = new;
		}
	}
	return;
}

void del_track(anUser *nick)
{
	struct track *current, *previous = NULL, *tmp;

	current = trackhead;
	while(current)
	{
		if(current->tracked == nick) break;
		else
		{
			previous = current;
			current = current->next;
		}
	}
	if(!current) return;
	else
	{
		if(current == trackhead) trackhead = trackhead->next;
		else previous->next = current->next;
		tmp = current->next;
		free(current);
		current = NULL;
	}
	return;
}

void cmd_track(aNick *nick, aChan *chaninfo, int parc, char **parv)
{
	struct track *track = NULL;
	if(!strcasecmp(parv[1], "list"))
	{
		unsigned int i=0;

		if(parc < 2) i=1;
		csntc(nick, "\2Liste des Traqus:\2");
		for(track=trackhead;track;track=track->next)
		{
			if(i || !match(parv[2],track->tracked->nick))
			{
				csntc(nick, "> \2%s\2 (Pour \2%s\2, depuis %s, expire %s)", track->tracked->nick, track->tracker->nick, duration((time(NULL) - track->from)), !track->expire ? "Jamais" : duration((time(NULL) - track->expire)));
			}
		}
		return;
	}
	if(!strcasecmp(parv[1],"add"))
	{
		anUser *user=NULL;

		if(parc < 2)
		{
			csntc(nick, "Syntaxe: TRACK ADD <username> [expiration]");
			return;
		}
		if(!(user = getuserinfo(parv[2])))
		{
			csntc(nick, NOSUCHUSER, parv[2]);
			return;
		}
		if((track = istrack(user)))
		{
			csntc(nick, "\2%s\2 est dj track par %s (depuis %s)", track->tracked->nick, track->tracker-nick, duration((time(NULL) - track->expire)));
			return;
		}
		if(nick->user == user)
		{
			csntc(nick, "Pourquoi se traquer soi-mme!?");
			return;
		}

		if(IsAnAdmin(user) && nick->user->level < 4)
		{
			csntc(nick, "Veuillez ne pas tracker un Admin.");
			return;
		}
		add_track(user, nick, (parc > 2 && is_num(parv[3])) ? atol(parv[3]) : 0);

		csntc(nick, "\2%s\2 a t ajout a vos tracks. (Expire dans %s)", user->nick, (parc > 2 && is_num(parv[3])) ? duration(time(NULL) - atol(parv[3])) : "Jamais");
		if(user->n) csntc(nick, "%s est actuellement connect via %s!%s@%s", user->nick, user->n->nick, user->n->ident, user->n->host);

		return;
	}
	if(!strcasecmp(parv[1],"del"))
	{
		anUser *user=NULL;
		if(parc < 2)
		{
			csntc(nick, "Syntaxe: TRACK DEL <username>");
			return;
		}
		if(!(user = getuserinfo(parv[2])))
		{
			csntc(nick, NOSUCHUSER, parv[2]);
			return;
		}
		if(!(track = istrack(user)))
		{
			csntc(nick, "\2%s\2 n'est pas track.", parv[2]);
			return;
		}
		if(track->tracker != nick)
		{
			csntc(nick, "\2%s\2 est actuellement track par %s.", track->tracked->nick, track->tracker->nick);
			return;
		}
		del_track(track->tracked);
		csntc(nick, "%s a t supprim de vos tracks.", parv[2]);
		return;
	}
	csntc(nick, "Commande invalide");
	return;
}
#endif
