#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <stdarg.h>
#include <fcntl.h>
#include "../include/web2cs.h"

#define RPL_102 "102 %s\n"


int bind_sock;
fd_set socks;        /* Socket file descriptors we want to wake up for, using select() */
fd_set master;
int highsock;        /* Highest #'d file descriptor, needed for select() */
struct w2c_clientinfo *w2c_clihead = NULL;
typedef struct w2c_clieninfo w2c_cli;

void w2c_initsocket(void)
{
	int port = 4567;
	int reuse_addr = 1;
	struct sockaddr_in server_address; /* bind info structure */


	bind_sock = socket(AF_INET, SOCK_STREAM, 0); /* on demande un socket */
	if (bind_sock < 0) {
		perror("socket");
		exit(EXIT_FAILURE);
	}
	fcntl(bind_sock, F_SETFL, O_NONBLOCK);

	setsockopt(bind_sock, SOL_SOCKET, SO_REUSEADDR, &reuse_addr, sizeof(reuse_addr));

	memset((char *) &server_address, 0, sizeof(server_address));
	server_address.sin_family = AF_INET;
	server_address.sin_addr.s_addr = INADDR_ANY;
	server_address.sin_port = htons(port);

	if (bind(bind_sock, (struct sockaddr *) &server_address, sizeof(server_address)) < 0 ) {
		perror("bind");
		close(bind_sock);
		exit(EXIT_FAILURE);
	}

	listen(bind_sock,5);

	highsock = bind_sock;
	FD_ZERO(&master);    // clear the master and temp sets
	FD_ZERO(&socks);

	FD_SET(bind_sock, &master);

	socks = master;
	return;
}

void w2c_checksocks(void)
{
	int i;
	struct sockaddr_in remoteaddr;
	int addrlen;
	int readsocks;
	struct timeval timeout;

	timeout.tv_sec = 4;
	timeout.tv_usec = 0;

	socks=master;
	readsocks = select(highsock+1, &socks, NULL, NULL, &timeout);

	if (readsocks == -1) {
		perror("select");
		exit(1);
	}
	else if (readsocks == 0) {
		//printf("Nothing to read ;)\n");
		return;
	}
	else  {
		//printf("Reading %d sockets\n", readsocks);
		w2c_readsocks();
	}
	return;
}



void w2c_readsocks(void)
{
	int i, addrlen, newfd, nbytes;
	char buf[256], ip[256];    // buffer for client data
	struct sockaddr_in remoteaddr; // client address
	struct w2c_clientinfo *cli;

	cli = w2c_clihead;

	for(i = 0;i <= highsock;i++)
	{
		if (FD_ISSET(i, &socks)) { // we got one!!
			if(i == bind_sock) { //Nouvelle connection !
				addrlen = sizeof(remoteaddr);
				if ((newfd = accept(bind_sock, (struct sockaddr *)&remoteaddr, &addrlen)) == -1) {
					perror("accept");
				} else {
					FD_SET(newfd, &master); // add to master set
					if (newfd > highsock) {    // keep track of the maximum
						highsock = newfd;
					}
					strcpy(ip, inet_ntoa(remoteaddr.sin_addr));
					w2c_addclient(newfd, ip);
					//printf("selectserver: new connection from %s on socket %d\n", inet_ntoa(remoteaddr.sin_addr), newfd);
				}
			}
			else { //Donnes d'un client
				if ((nbytes = recv(i, buf, sizeof(buf), 0)) <= 0) {
					// got error or connection closed by client
					if (nbytes == 0) {
						// connection closed
						//printf("selectserver: socket %d hung up\n", i);
					} else {
						perror("recv");
					}
					cli = w2c_findclient(i);
					w2c_delclient(cli);
					close(i); // bye!
					FD_CLR(i, &master); // remove from master set
				} else {
					// we got some data from a client
					cli = w2c_findclient(i);
					w2c_parsemsg(cli, buf);
				}
			}
		}
	}
	return;
}

void w2c_addclient(int i, char *a1)
{
	struct w2c_clientinfo *w2c, *new;

	new = (struct w2c_clientinfo *) calloc(0, sizeof(struct w2c_clientinfo));
	if(!new) return;

	new->next = NULL;
	new->socket = i;
	new->ip[0] = '\0';

	strcpy(new->ip, a1);
	new->flag = 0;
	if(!w2c_clihead) w2c_clihead = new;
	else {
		new->next = w2c_clihead;
		w2c_clihead = new;
	}

	return;
}

void w2c_delclient(struct w2c_clientinfo *del)
{
	struct w2c_clientinfo *last, *next;

	next = w2c_clihead;
	while(next)
	{
		if(next == del) break;
		last = next;
		next = next->next;
	}

	last->next = next->next;
	
	free(next);
	next = NULL;
	return;
}

struct w2c_clientinfo *w2c_findclient(int i)
{
	struct w2c_clientinfo *cli = w2c_clihead;

	while(cli)
	{
		if(cli->socket == i) return cli;
		cli = cli->next;
	}
	return NULL;
}

void w2c_parsemsg(struct w2c_clientinfo *source, char *msg)
{
	char *parv[256], buf[1024], *ptr, *ptr2, *cmd;
	int parc = 1, i = 0;

	if((ptr2 = strchr(msg, '\n'))) ptr2 = '\0';
	if((ptr2 = strchr(msg, '\r'))) *ptr2 = '\0';


	strcpy(buf, msg);

	cmd = strtok(buf, " ");
	if(!cmd)
		cmd = msg;
	else
		for(; (ptr=strtok(NULL, " ")) && parc < 256 ; parc++)
			parv[parc] = ptr;

	parc = parc - 1;

	for (i=0; w2c_cmds[i].func != NULL; i++)
	{
		if (!strcasecmp(w2c_cmds[i].cmd,cmd))
		{
			if(parc < w2c_cmds[i].args)
			{
				w2c_sendrpl(source, RPL_102, cmd);
				return;
			}
			w2c_cmds[i].func(source, parc, parv);
			return;
		}
	}

	return;
}

/*
 * Dbut : parv[1]
 * Fin : parv[parc]
 */

void w2c_sendrpl(struct w2c_clientinfo *dest, const char *pattern, ...)
{
	char buf[1024];
	va_list vl;
	int len;
	buf[0]='\0';

	va_start(vl, pattern);
	vsnprintf(buf, 1024, pattern, vl);
	va_end(vl);

	len = strlen(buf);

	if (send(dest->socket, buf, len, 0) == -1) {
		perror("send");
	}
	return;
}

void w2c_login(struct w2c_clientinfo *source, int parc, char **parv)
{

	return;

}

void w2c_pong(struct w2c_clientinfo *source, int parc, char **parv) { return; }
void w2c_moduser(struct w2c_clientinfo *source, int parc, char **parv) { return; }
void w2c_chaninfo(struct w2c_clientinfo *source, int parc, char **parv) { return; }
void w2c_access(struct w2c_clientinfo *source, int parc, char **parv) { return; }
void w2c_set(struct w2c_clientinfo *source, int parc, char **parv) { return; }
void w2c_user(struct w2c_clientinfo *source, int parc, char **parv) { return; }
void w2c_chan(struct w2c_clientinfo *source, int parc, char **parv) { return; }

int main(void)
{
	w2c_initsocket();

	for(;;)
		w2c_checksocks();

	return 1;
}



